@@ -1,6 +1,6 @@
 #include <Rcpp.h>
-#include "barry/barry.hpp"
-#include "barry/models/defm.hpp"
+#include <barry/barry.hpp>
+#include <barry/models/defm.hpp>
 
 using namespace Rcpp;
 
@@ -15,7 +15,25 @@ using namespace Rcpp;
 //' @param Y 0/1 matrix of responses of `n_y` columns and `n` rows.
 //' @param X Numeric matrix of covariates of size `n_x` by `n`.
 //' @param order Integer. Order of the markov process, by default, 1.
+//' @param copy_data Logical scalar. When `TRUE` (default) will copy the data
+//' into the model, otherwise it will use the data as a pointer (see details).
 //'
+//' @details
+//' The `id` vector is used to group the observations. For example, if you have
+//' a dataset with multiple individuals, the `id` vector should contain the
+//' individual ids. The `Y` matrix contains the binary responses, where each
+//' column represents a different response variable. The `X` matrix contains
+//' the covariates, which can be used to model the relationship between the
+//' responses and the covariates. The `order` parameter specifies the order of
+//' the Markov process, which determines how many previous observations are
+//' used to predict the current observation.
+//'
+//' The `copy_data` parameter specifies
+//' whether the data should be copied into the model or used as a pointer. If
+//' `copy_data` is `TRUE`, the data will be copied into the model, which can
+//' be useful if you want to avoid duplicating the data in memory. If 
+//' `copy_data` is `FALSE`, the model will use the data as a pointer, which can 
+//' be more efficient (but dangerous if the data is removed).
 //' @return An external pointer of class `DEFM.`
 //'
 //' @name DEFM
@@ -32,14 +50,14 @@ using namespace Rcpp;
 //' )
 //' 
 //' # Adding the intercept terms and a motif from tobacco to mj
-//' term_defm_logit_intercept(mymodel)
-//' term_defm_transition_formula(mymodel, "{y1, 0y2} > {y1, y2}")
+//' td_logit_intercept(mymodel)
+//' td_formula(mymodel, "{y1, 0y2} > {y1, y2}")
 //' 
 //' # Initialize the model
 //' init_defm(mymodel)
 //' 
 //' # Fitting the MLE
-/// defm_mle(mymodel)
+//' defm_mle(mymodel)
 //' 
 // [[Rcpp::export(rng = false, name = 'new_defm_cpp')]]
 SEXP new_defm(
@@ -50,6 +68,18 @@ SEXP new_defm(
     bool copy_data = true
   ) {
 
+  // Adding typechecks to see if the SEXP objects have
+  // the class attribute equal to the type
+  if (!Rf_isInteger(id)) {
+    stop("id must be an integer vector");
+  }
+  if (!Rf_isMatrix(Y) || !Rf_isInteger(Y)) {
+    stop("Y must be an integer matrix");
+  }
+  if (!Rf_isMatrix(X) || !Rf_isNumeric(X)) {
+    stop("X must be a numeric matrix");
+  }
+
   int n_id = LENGTH(id);
   int n_y  = Rf_ncols(Y);
   int n_x  = Rf_ncols(X);
@@ -144,12 +174,21 @@ CharacterVector get_X_names(
 
 //' @rdname DEFM
 //' @param m An object of class `DEFM`.
+//' @param force_new Logical scalar. When `TRUE` (default) no cache is used
+//' to add new arrays (see details).
+//' @details
+//' The `init_defm` function initializes the model, which means it computes
+//' the sufficient statistics and prepares the model for fitting. The 
+//' `force_new` parameter specifies whether to force the model to be 
+//' consider each array added as completely unique, even if it has the
+//' same support set as an existing array. This is an experimental feature
+//' and should be used with caution.  
 //' @export
 // [[Rcpp::export(invisible = true, rng = false)]]
-SEXP init_defm(SEXP m)
+SEXP init_defm(SEXP m, bool force_new = false)
 {
   Rcpp::XPtr< defm::DEFM > ptr(m);
-  ptr->init();
+  ptr->init(force_new);
   return m;
 }
 
@@ -188,8 +227,8 @@ SEXP print_defm(SEXP x)
 //' )
 //' 
 //' # Adding the intercept terms and a motif from tobacco to mj
-//' term_defm_logit_intercept(mymodel)
-//' term_defm_transition_formula(mymodel, "{y1, 0y2} > {y1, y2}")
+//' td_logit_intercept(mymodel)
+//' td_formula(mymodel, "{y1, 0y2} > {y1, y2}")
 //'
 //' # Computing the log-likelihood
 //' loglike_defm(mymodel, par = c(-1, -1, -1, 2), as_log = TRUE)
@@ -269,6 +308,10 @@ IntegerMatrix sim_defm(
 
   }
 
+  // Adding a name
+  CharacterVector names = wrap(ptr->get_Y_names());
+  Rcpp::colnames(res) = names;
+
   return res;
 }
 
@@ -299,6 +342,7 @@ int nterms_defm(SEXP m)
   return ptr->nterms();
 }
 
+//' @export
 // [[Rcpp::export(rng = false, name = "names.DEFM")]]
 CharacterVector names_defm(SEXP x)
 {
@@ -389,8 +433,8 @@ int morder_defm(SEXP m)
 //' )
 //' 
 //' # Adding the intercept terms and a motif from tobacco to mj
-//' term_defm_logit_intercept(mymodel)
-//' term_defm_transition_formula(mymodel, "{y1, 0y2} > {y1, y2}")
+//' td_logit_intercept(mymodel)
+//' td_formula(mymodel, "{y1, 0y2} > {y1, y2}")
 //' 
 //' # Initialize the model
 //' init_defm(mymodel)
