@@ -8,7 +8,9 @@ using namespace Rcpp;
 //' Model specification for DEFM
 //'
 //' @param m An object of class [DEFM].
-//' @param idx Integer. When greater than -1, index (starting from 0) of a covariate
+//' @param covar String. Name of a covariate to use as an interaction
+//' for the effect. If equal to `""`, then no interaction effect.
+//' is used.
 //' used to weight the term.
 //' @export
 //'
@@ -27,53 +29,48 @@ using namespace Rcpp;
 //'   order = 1
 //' )
 //' 
-//' # Adding the intercept terms and a motif from tobacco to mj
-//' term_defm_logit_intercept(mymodel)
-//' term_defm_transition_formula(mymodel, "{y1, 0y2} > {y1, y2}")
+//' # Conventional regression intercept
+//' td_logit_intercept(mymodel)
+//'
+//' # Interaction effect with Hispanic
+//' td_logit_intercept(mymodel, covar = "Hispanic")
+//' 
+//' # Transition effect from only y1 to both equal to 1.
+//' td_formula(mymodel, "{y1, 0y2} > {y1, y2}")
+//' 
+//' # Same but interaction with Female
+//' td_formula(mymodel, "{y1, 0y2} > {y1, y2} x Female")
 //'
 //' # Inspecting the model
 //' mymodel
+//' 
+//' # Initializing and fitting
+//' init_defm(mymodel)
+//' defm_mle(mymodel)
 // [[Rcpp::export(invisible = true, rng = false)]]
-SEXP term_defm_ones(SEXP m, std::string idx = "", std::string vname = "")
-{
+SEXP td_ones(
+  SEXP m,
+  std::string covar = ""
+) {
 
   int idx_ = -1;
   Rcpp::XPtr< defm::DEFM > ptr(m);
 
   // This will set the covar index, if needed
-  check_covar(idx_, idx, ptr);
+  check_covar(idx_, covar, ptr);
 
   defm::counter_ones(
-    ptr->get_counters(), idx_, vname,
+    ptr->get_counters(), idx_,
     &ptr->get_X_names()
     );
 
   return m;
 }
 
-
-//' @rdname defm_terms
-//' @export
-//' @param k Numeric scalar. Exponent used in the term.
-// [[Rcpp::export(invisible = true, rng = false)]]
-SEXP term_defm_fe(SEXP m, std::string idx = "", double k = 1.0, std::string vname = "")
-{
-
-  Rcpp::XPtr< defm::DEFM > ptr(m);
-  int idx_ = -1;
-
-  // This will set the covar index, if needed
-  check_covar(idx_, idx, ptr);
-
-  defm::counter_fixed_effect(ptr->get_counters(), idx_, k, vname);
-
-  return m;
-}
-
 //' @param mat Integer matrix. The matrix specifies the type of motif to capture
 //' (see details.)
 //' @details
-//' In `term_defm_transition`, users can specify a particular motif to model. Motifs
+//' In `td_generic`, users can specify a particular motif to model. Motifs
 //' are represented by cells with values equal to 1, for example, the matrix:
 //'
 //' ```  y0 y1 y2
@@ -86,11 +83,10 @@ SEXP term_defm_fe(SEXP m, std::string idx = "", double k = 1.0, std::string vnam
 //' @export
 //' @rdname defm_terms
 // [[Rcpp::export(invisible = true, rng = false)]]
-SEXP term_defm_transition(
+SEXP td_generic(
     SEXP m,
     IntegerMatrix & mat,
-    std::string idx = "",
-    std::string vname = ""
+    std::string covar = ""
 )
 {
 
@@ -98,7 +94,7 @@ SEXP term_defm_transition(
   int idx_ = -1;
 
   // This will set the covar index, if needed
-  check_covar(idx_, idx, ptr);
+  check_covar(idx_, covar, ptr);
 
   if (static_cast<size_t>(mat.nrow()) != (ptr->get_m_order() + 1u))
     stop("The number of rows in -mat- must be equal to the Markov order of the model + 1.");
@@ -134,10 +130,10 @@ SEXP term_defm_transition(
 
   }
 
-    defm::counter_transition(
+    defm::counter_generic(
       ptr->get_counters(), coords, signs,
       ptr->get_m_order(), ptr->get_n_y(),
-      idx_, vname,
+      idx_,
       &ptr->get_X_names(),
       &ptr->get_Y_names()
     );
@@ -147,23 +143,30 @@ SEXP term_defm_transition(
 }
 
 
-//' @details The function `term_defm_transition_formula`,
+//' @details The function `td_formula`,
 //' will take the formula and generate the corresponding
 //' input for defm::counter_transition(). Formulas can be specified in the
 //' following ways:
 //'
-//' - Intercept effect: {...} No transition, only including the current state.
-//' - Transition effect: {...} > {...} Includes current and previous states.
+//' - Intercept effect: `{...}` No transition, only including the current state.
+//' - Transition effect: `{...} > {...}` Includes current and previous states.
 //'
 //' The general notation is `[0]y[column id]_[row id]`. A preceeding zero
 //' means that the value of the cell is considered to be zero. The column
 //' id goes between 0 and the number of columns in the array - 1 (so it
 //' is indexed from 0,) and the row id goes from 0 to m_order.
 //'
+//' Both Intercepts and Transition can interact with covariates. Using 
+//' either the `covar` argument or, in the case of formulas, `x [Covar name]`,
+//' for example:
+//'
+//' - Intercept effect: `{...} x Hispanic` interacts with the Hispanic covar.
+//' - Transition effect: `{...} > {...} x Hispanic` Same.
+//'
 //' ## Intercept effects
 //'
 //' Intercept effects only involve a single set of curly brackets. Using the
-//' 'greater-than' symbol (i.e., '<') is only for transition effects. When
+//' 'greater-than' symbol (i.e., `<`) is only for transition effects. When
 //' specifying intercept effects, users can skip the `row_id`, e.g.,
 //' `y0_0` is equivalent to `y0`. If the passed `row id` is different from
 //' the Markov order, i.e., `row_id != m_order`, then the function returns
@@ -180,64 +183,70 @@ SEXP term_defm_transition(
 //' an greater-than symbol, i.e., `{...} > {...}`. The first set of brackets,
 //' which we call LHS, can only hold `row id` that are less than `m_order`.
 //' @param formula Character scalar (see details).
-//' @param idx Character scalar. Name of the variable to include in the term.
-//' @param vname Character scalar. Name to be assigned for the new term.
+//' @param new_name Character scalar. Name to be assigned for the new term.
+//' if empty, then it builds a name based on the formula.
 //' @export
 //' @rdname defm_terms
 // [[Rcpp::export(invisible = true, rng = false)]]
-SEXP term_defm_transition_formula(
+SEXP td_formula(
     SEXP m,
     std::string formula,
-    std::string idx = "",
-    std::string vname = ""
+    std::string new_name = ""
 )
 {
 
   Rcpp::XPtr< defm::DEFM > ptr(m);
 
-  int idx_ = -1;
-
-  // This will set the covar index, if needed
-  check_covar(idx_, idx, ptr);
-
-  defm::counter_transition_formula(
+  defm::counter_formula(
     ptr->get_counters(), formula,
-    ptr->get_m_order(), ptr->get_n_y(),
-    idx_, vname,
+    ptr->get_m_order(),
+    ptr->get_n_y(),
     &ptr->get_X_names(),
     &ptr->get_Y_names()
   );
 
+  // Renaming the counter on the fly (if the name is too long)
+  if (new_name != "")
+  {
+    auto n_counter = ptr->nterms();
+
+    if (n_counter == 0)
+      stop("Something went wrong. No terms in the model.");
+
+    (*ptr->get_counters())[n_counter - 1].set_name(
+      new_name
+    );
+  }
+
   return m;
 
 }
 
 //' @export
 //' @rdname defm_terms
-//' @details The term `term_defm_logit_intercept` will add what is equivalent to an
-//' intercept in a logistic regression. When `coords` is specified, then the
+//' @details The term `td_logit_intercept` will add what is equivalent to an
+//' intercept in a logistic regression. When `y_indices` is specified, then the
 //' function will add one intercept per outcome. These can be weighted by
 //' a covariate.
-//' @param coords Integer vector with the coordinates to include in the term.
+//' @param y_indices Integer vector with the coordinates to include in the term.
 // [[Rcpp::export(invisible = true, rng = false)]]
-SEXP term_defm_logit_intercept(
+SEXP td_logit_intercept(
   SEXP m,
-  IntegerVector coords = IntegerVector::create(),
-  std::string idx = "",
-  std::string vname = ""
+  IntegerVector y_indices = IntegerVector::create(),
+  std::string covar = ""
 ) {
 
   Rcpp::XPtr< defm::DEFM > ptr(m);
   int idx_ = -1;
 
   // This will set the covar index, if needed
-  check_covar(idx_, idx, ptr);
+  check_covar(idx_, covar, ptr);
 
   std::vector< size_t > coords_;
-  for (auto c : coords)
+  for (auto c : y_indices)
   {
     if (c < 0)
-      stop("Element in coords is negative. Only zero or positive are allowed");
+      stop("Element in `y_indices` is negative. Only zero or positive are allowed");
     coords_.push_back(c);
   }
 
@@ -246,7 +255,6 @@ SEXP term_defm_logit_intercept(
     ptr->get_n_y(),
     coords_,
     idx_,
-    vname,
     &ptr->get_X_names(),
     &ptr->get_Y_names()
   );
@@ -259,19 +267,52 @@ SEXP term_defm_logit_intercept(
 //' @details The function `rule_not_one_to_zero` will avoid the transition one to zero in a Markov process.
 //' @export
 //' @rdname defm_terms
+//' @param term_indices Non-negative vector of indices. Indicates which
+//' outcomes this rule will apply.
 // [[Rcpp::export(invisible = true, rng = false)]]
 SEXP rule_not_one_to_zero(
   SEXP m,
-  std::vector< size_t > idx
+  std::vector< size_t > term_indices
 ) {
 
   Rcpp::XPtr< defm::DEFM > ptr(m);
 
   defm::rules_dont_become_zero(
     ptr->get_support_fun(),
-    idx
+    term_indices
   );
 
   return m;
 }
 
+//' @details The function `rule_constrain_support` will constrain the support of the model
+//' by specifying a lower and upper bound for a given statistic.
+//' @param lb,ub Numeric scalars. Lower and upper bounds.
+//' @param term_index Non-negative scalar. Which term this rule will apply.
+//' @rdname defm_terms
+//' @export
+// [[Rcpp::export(invisible = true, rng = false)]]
+SEXP rule_constrain_support(
+  SEXP m,
+  int term_index,
+  double lb,
+  double ub
+) {
+
+  if (term_index < 0)
+    stop("`term_index` must be greater than or equal to zero.");
+
+  Rcpp::XPtr< defm::DEFM > ptr(m);
+
+  defm::rule_constrain_support(
+    ptr->get_support_fun(),
+    static_cast<size_t>(term_index),
+    lb,
+    ub
+  );
+
+  return m;
+
+
+
+}
